@@ -1,4 +1,4 @@
-# Copyright (C) 2021-2022 Free Software Foundation, Inc.
+# Copyright (C) 2021-2026 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,11 +13,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import gdb
-import gdb.disassembler
 import struct
 import sys
 
+import gdb
+import gdb.disassembler
 from gdb.disassembler import Disassembler, DisassemblerResult
 
 # A global, holds the program-counter address at which we should
@@ -25,8 +25,26 @@ from gdb.disassembler import Disassembler, DisassemblerResult
 current_pc = None
 
 
+def builtin_disassemble_wrapper(info):
+    result = gdb.disassembler.builtin_disassemble(info)
+    assert result.length > 0
+    assert len(result.parts) > 0
+    tmp_str = ""
+    for p in result.parts:
+        assert p.string == str(p)
+        tmp_str += p.string
+    assert tmp_str == result.string
+    return result
+
+
+def check_building_disassemble_result():
+    """Check that we can create DisassembleResult objects correctly."""
+    gdb.disassembler.DisassemblerResult()
+    print("PASS")
+
+
 def is_nop(s):
-    return s == "nop" or s == "nop\t0"
+    return s == "nop" or s == "nop\t0" or s == "nop\t{0}"
 
 
 # Remove all currently registered disassemblers.
@@ -42,16 +60,12 @@ class TestDisassembler(Disassembler):
     disassembly wrapping for the global CURRENT_PC."""
 
     def __init__(self):
-        global current_pc
-
         super().__init__("TestDisassembler")
         self.__info = None
-        if current_pc == None:
+        if current_pc is None:
             raise gdb.GdbError("no current_pc set")
 
     def __call__(self, info):
-        global current_pc
-
         if info.address != current_pc:
             return None
         self.__info = info
@@ -64,6 +78,67 @@ class TestDisassembler(Disassembler):
         raise NotImplementedError("override the disassemble method")
 
 
+class ShowInfoRepr(TestDisassembler):
+    """Call the __repr__ method on the DisassembleInfo, convert the result
+    to a string, and incude it in a comment in the disassembler output."""
+
+    def disassemble(self, info):
+        comment = "\t## " + repr(info)
+        result = builtin_disassemble_wrapper(info)
+        string = result.string + comment
+        length = result.length
+        return DisassemblerResult(length=length, string=string)
+
+
+class ShowInfoSubClassRepr(TestDisassembler):
+    """Create a sub-class of DisassembleInfo.  Create an instace of this
+    sub-class and call the __repr__ method on it.  Convert the result
+    to a string, and incude it in a comment in the disassembler
+    output.  The DisassembleInfo sub-class does not override __repr__
+    so we are calling the implementation on the parent class."""
+
+    class MyInfo(gdb.disassembler.DisassembleInfo):
+        """A wrapper around DisassembleInfo, doesn't add any new
+        functionality, just gives a new name in order to check the
+        __repr__ functionality."""
+
+        def __init__(self, info):
+            super().__init__(info)
+
+    def disassemble(self, info):
+        info = self.MyInfo(info)
+        comment = "\t## " + repr(info)
+        result = builtin_disassemble_wrapper(info)
+        string = result.string + comment
+        length = result.length
+        return DisassemblerResult(length=length, string=string)
+
+
+class ShowResultRepr(TestDisassembler):
+    """Call the __repr__ method on the DisassemblerResult, convert the
+    result to a string, and incude it in a comment in the disassembler
+    output."""
+
+    def disassemble(self, info):
+        result = builtin_disassemble_wrapper(info)
+        comment = "\t## " + repr(result)
+        string = result.string + comment
+        length = result.length
+        return DisassemblerResult(length=length, string=string)
+
+
+class ShowResultStr(TestDisassembler):
+    """Call the __str__ method on a DisassemblerResult object, incude the
+    resulting string in a comment within the disassembler output."""
+
+    def disassemble(self, info):
+        result = builtin_disassemble_wrapper(info)
+        comment = "\t## " + str(result)
+        string = result.string + comment
+        length = result.length
+        return DisassemblerResult(length=length, string=string, parts=None)
+
+
 class GlobalPreInfoDisassembler(TestDisassembler):
     """Check the attributes of DisassembleInfo before disassembly has occurred."""
 
@@ -77,7 +152,7 @@ class GlobalPreInfoDisassembler(TestDisassembler):
         if not isinstance(ar, gdb.Architecture):
             raise gdb.GdbError("invalid architecture type")
 
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
 
         text = result.string + "\t## ad = 0x%x, ar = %s" % (ad, ar.name())
         return DisassemblerResult(result.length, text)
@@ -87,7 +162,7 @@ class GlobalPostInfoDisassembler(TestDisassembler):
     """Check the attributes of DisassembleInfo after disassembly has occurred."""
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
 
         ad = info.address
         ar = info.architecture
@@ -108,7 +183,7 @@ class GlobalReadDisassembler(TestDisassembler):
     adds them as a comment to the disassembler output."""
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
         len = result.length
         str = ""
         for o in range(len):
@@ -126,7 +201,7 @@ class GlobalAddrDisassembler(TestDisassembler):
     """Check the gdb.format_address method."""
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
         arch = info.architecture
         addr = info.address
         program_space = info.progspace
@@ -153,7 +228,7 @@ class GdbErrorLateDisassembler(TestDisassembler):
     """Raise a GdbError after calling the builtin disassembler."""
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
+        builtin_disassemble_wrapper(info)
         raise gdb.GdbError("GdbError after builtin disassembler")
 
 
@@ -161,7 +236,7 @@ class RuntimeErrorLateDisassembler(TestDisassembler):
     """Raise a RuntimeError after calling the builtin disassembler."""
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
+        builtin_disassemble_wrapper(info)
         raise RuntimeError("RuntimeError after builtin disassembler")
 
 
@@ -171,10 +246,10 @@ class MemoryErrorEarlyDisassembler(TestDisassembler):
     def disassemble(self, info):
         tag = "## FAIL"
         try:
-            info.read_memory(1, -info.address + 2)
+            info.read_memory(1, -info.address - 1)
         except gdb.MemoryError:
             tag = "## AFTER ERROR"
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
         text = result.string + "\t" + tag
         return DisassemblerResult(result.length, text)
 
@@ -184,9 +259,9 @@ class MemoryErrorLateDisassembler(TestDisassembler):
     before we return a result."""
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
+        builtin_disassemble_wrapper(info)
         # The following read will throw an error.
-        info.read_memory(1, -info.address + 2)
+        info.read_memory(1, -info.address - 1)
         return DisassemblerResult(1, "BAD")
 
 
@@ -195,9 +270,9 @@ class RethrowMemoryErrorDisassembler(TestDisassembler):
 
     def disassemble(self, info):
         try:
-            info.read_memory(1, -info.address + 2)
-        except gdb.MemoryError as e:
-            raise gdb.MemoryError("cannot read code at address 0x2")
+            info.read_memory(1, -info.address - 1)
+        except gdb.MemoryError:
+            raise gdb.MemoryError("cannot read code at address -1")
         return DisassemblerResult(1, "BAD")
 
 
@@ -213,41 +288,22 @@ class ResultOfWrongType(TestDisassembler):
         return self.Blah(1, "ABC")
 
 
-class ResultWrapper(gdb.disassembler.DisassemblerResult):
-    def __init__(self, length, string, length_x=None, string_x=None):
-        super().__init__(length, string)
-        if length_x is None:
-            self.__length = length
-        else:
-            self.__length = length_x
-        if string_x is None:
-            self.__string = string
-        else:
-            self.__string = string_x
+class ResultOfVeryWrongType(TestDisassembler):
+    """Return something that is not a DisassemblerResult from disassemble
+    method.  The thing returned will raise an exception if used in an
+    isinstance() call, or in PyObject_IsInstance from C++.
+    """
 
-    @property
-    def length(self):
-        return self.__length
+    class Blah:
+        def __init__(self):
+            pass
 
-    @property
-    def string(self):
-        return self.__string
-
-
-class ResultWithInvalidLength(TestDisassembler):
-    """Return a result object with an invalid length."""
+        @property
+        def __class__(self):
+            raise RuntimeError("error from __class__ in Blah")
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
-        return ResultWrapper(result.length, result.string, 0)
-
-
-class ResultWithInvalidString(TestDisassembler):
-    """Return a result object with an empty string."""
-
-    def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
-        return ResultWrapper(result.length, result.string, None, "")
+        return self.Blah()
 
 
 class TaggingDisassembler(TestDisassembler):
@@ -258,7 +314,7 @@ class TaggingDisassembler(TestDisassembler):
         self._tag = tag
 
     def disassemble(self, info):
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
         text = result.string + "\t## tag = %s" % self._tag
         return DisassemblerResult(result.length, text)
 
@@ -283,7 +339,7 @@ class GlobalCachingDisassembler(TestDisassembler):
         and cache the DisassembleInfo so that it is not garbage collected."""
         GlobalCachingDisassembler.cached_insn_disas.append(info)
         GlobalCachingDisassembler.cached_insn_disas.append(self.MyInfo(info))
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
         text = result.string + "\t## CACHED"
         return DisassemblerResult(result.length, text)
 
@@ -295,31 +351,31 @@ class GlobalCachingDisassembler(TestDisassembler):
             assert isinstance(info, gdb.disassembler.DisassembleInfo)
             assert not info.is_valid()
             try:
-                val = info.address
+                info.address
                 raise gdb.GdbError("DisassembleInfo.address is still valid")
             except RuntimeError as e:
                 assert str(e) == "DisassembleInfo is no longer valid."
-            except:
+            except Exception:
                 raise gdb.GdbError(
                     "DisassembleInfo.address raised an unexpected exception"
                 )
 
             try:
-                val = info.architecture
+                info.architecture
                 raise gdb.GdbError("DisassembleInfo.architecture is still valid")
             except RuntimeError as e:
                 assert str(e) == "DisassembleInfo is no longer valid."
-            except:
+            except Exception:
                 raise gdb.GdbError(
                     "DisassembleInfo.architecture raised an unexpected exception"
                 )
 
             try:
-                val = info.read_memory(1, 0)
+                info.read_memory(1, 0)
                 raise gdb.GdbError("DisassembleInfo.read is still valid")
             except RuntimeError as e:
                 assert str(e) == "DisassembleInfo is no longer valid."
-            except:
+            except Exception:
                 raise gdb.GdbError(
                     "DisassembleInfo.read raised an unexpected exception"
                 )
@@ -349,7 +405,7 @@ class ReadMemoryMemoryErrorDisassembler(TestDisassembler):
 
     def disassemble(self, info):
         info = self.MyInfo(info)
-        return gdb.disassembler.builtin_disassemble(info)
+        return builtin_disassemble_wrapper(info)
 
 
 class ReadMemoryGdbErrorDisassembler(TestDisassembler):
@@ -365,7 +421,7 @@ class ReadMemoryGdbErrorDisassembler(TestDisassembler):
 
     def disassemble(self, info):
         info = self.MyInfo(info)
-        return gdb.disassembler.builtin_disassemble(info)
+        return builtin_disassemble_wrapper(info)
 
 
 class ReadMemoryRuntimeErrorDisassembler(TestDisassembler):
@@ -381,7 +437,7 @@ class ReadMemoryRuntimeErrorDisassembler(TestDisassembler):
 
     def disassemble(self, info):
         info = self.MyInfo(info)
-        return gdb.disassembler.builtin_disassemble(info)
+        return builtin_disassemble_wrapper(info)
 
 
 class ReadMemoryCaughtMemoryErrorDisassembler(TestDisassembler):
@@ -398,7 +454,7 @@ class ReadMemoryCaughtMemoryErrorDisassembler(TestDisassembler):
     def disassemble(self, info):
         info = self.MyInfo(info)
         try:
-            return gdb.disassembler.builtin_disassemble(info)
+            return builtin_disassemble_wrapper(info)
         except gdb.MemoryError:
             return None
 
@@ -417,7 +473,7 @@ class ReadMemoryCaughtGdbErrorDisassembler(TestDisassembler):
     def disassemble(self, info):
         info = self.MyInfo(info)
         try:
-            return gdb.disassembler.builtin_disassemble(info)
+            return builtin_disassemble_wrapper(info)
         except gdb.GdbError as e:
             if e.args[0] == "exception message":
                 return None
@@ -438,7 +494,7 @@ class ReadMemoryCaughtRuntimeErrorDisassembler(TestDisassembler):
     def disassemble(self, info):
         info = self.MyInfo(info)
         try:
-            return gdb.disassembler.builtin_disassemble(info)
+            return builtin_disassemble_wrapper(info)
         except RuntimeError as e:
             if e.args[0] == "exception message":
                 return None
@@ -455,7 +511,7 @@ class MemorySourceNotABufferDisassembler(TestDisassembler):
 
     def disassemble(self, info):
         info = self.MyInfo(info)
-        return gdb.disassembler.builtin_disassemble(info)
+        return builtin_disassemble_wrapper(info)
 
 
 class MemorySourceBufferTooLongDisassembler(TestDisassembler):
@@ -477,7 +533,95 @@ class MemorySourceBufferTooLongDisassembler(TestDisassembler):
 
     def disassemble(self, info):
         info = self.MyInfo(info)
-        return gdb.disassembler.builtin_disassemble(info)
+        return builtin_disassemble_wrapper(info)
+
+
+class ErrorCreatingTextPart_NoArgs(TestDisassembler):
+    """Try to create a DisassemblerTextPart with no arguments."""
+
+    def disassemble(self, info):
+        info.text_part()
+
+
+class ErrorCreatingAddressPart_NoArgs(TestDisassembler):
+    """Try to create a DisassemblerAddressPart with no arguments."""
+
+    def disassemble(self, info):
+        info.address_part()
+
+
+class ErrorCreatingTextPart_NoString(TestDisassembler):
+    """Try to create a DisassemblerTextPart with no string argument."""
+
+    def disassemble(self, info):
+        info.text_part(gdb.disassembler.STYLE_TEXT)
+
+
+class ErrorCreatingTextPart_NoStyle(TestDisassembler):
+    """Try to create a DisassemblerTextPart with no string argument."""
+
+    def disassemble(self, info):
+        info.text_part(string="abc")
+
+
+class ErrorCreatingTextPart_StringAndParts(TestDisassembler):
+    """Try to create a DisassemblerTextPart with both a string and a parts list."""
+
+    def disassemble(self, info):
+        parts = []
+        parts.append(info.text_part(gdb.disassembler.STYLE_TEXT, "p1"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_TEXT, "p2"))
+
+        return DisassemblerResult(length=4, string="p1p2", parts=parts)
+
+
+class All_Text_Part_Styles(TestDisassembler):
+    """Create text parts with all styles."""
+
+    def disassemble(self, info):
+        parts = []
+        parts.append(info.text_part(gdb.disassembler.STYLE_TEXT, "p1"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_MNEMONIC, "p2"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_SUB_MNEMONIC, "p3"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_ASSEMBLER_DIRECTIVE, "p4"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_REGISTER, "p5"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_IMMEDIATE, "p6"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_ADDRESS, "p7"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_ADDRESS_OFFSET, "p8"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_SYMBOL, "p9"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_COMMENT_START, "p10"))
+
+        result = builtin_disassemble_wrapper(info)
+        result = DisassemblerResult(length=result.length, parts=parts)
+
+        tmp_str = ""
+        for p in parts:
+            assert p.string == str(p)
+            tmp_str += str(p)
+        assert tmp_str == result.string
+
+        return result
+
+
+class Build_Result_Using_All_Parts(TestDisassembler):
+    """Disassemble an instruction and return a result that makes use of
+    text and address parts."""
+
+    def disassemble(self, info):
+        parts = []
+        parts.append(info.text_part(gdb.disassembler.STYLE_MNEMONIC, "fake"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_TEXT, "\t"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_REGISTER, "reg"))
+        parts.append(info.text_part(gdb.disassembler.STYLE_TEXT, ", "))
+        addr_part = info.address_part(current_pc)
+        assert addr_part.address == current_pc
+        parts.append(addr_part)
+        parts.append(info.text_part(gdb.disassembler.STYLE_TEXT, ", "))
+        parts.append(info.text_part(gdb.disassembler.STYLE_IMMEDIATE, "123"))
+
+        result = builtin_disassemble_wrapper(info)
+        result = DisassemblerResult(length=result.length, parts=parts)
+        return result
 
 
 class BuiltinDisassembler(Disassembler):
@@ -487,7 +631,7 @@ class BuiltinDisassembler(Disassembler):
         super().__init__("BuiltinDisassembler")
 
     def __call__(self, info):
-        return gdb.disassembler.builtin_disassemble(info)
+        return builtin_disassemble_wrapper(info)
 
 
 class AnalyzingDisassembler(Disassembler):
@@ -582,7 +726,7 @@ class AnalyzingDisassembler(Disassembler):
         # Override the info object, this provides access to our
         # read_memory function.
         info = self.MyInfo(info, self._start, self._end, self._nop_bytes)
-        result = gdb.disassembler.builtin_disassemble(info)
+        result = builtin_disassemble_wrapper(info)
 
         # Record some informaiton about the first 'nop' instruction we find.
         if self._nop_index is None and is_nop(result.string):
@@ -621,8 +765,8 @@ class AnalyzingDisassembler(Disassembler):
                 idx > 0
                 and idx != nop_idx
                 and not is_nop(self._pass_1_insn[idx])
-                and self._pass_1_length[idx] > self._pass_1_length[nop_idx]
-                and self._pass_1_length[idx] % self._pass_1_length[nop_idx] == 0
+                and self._pass_1_length[idx] > nop_length
+                and self._pass_1_length[idx] % nop_length == 0
             ):
                 replace_idx = idx
                 break
@@ -636,7 +780,7 @@ class AnalyzingDisassembler(Disassembler):
                     idx > 0
                     and idx != nop_idx
                     and not is_nop(self._pass_1_insn[idx])
-                    and self._pass_1_length[idx] == self._pass_1_length[nop_idx]
+                    and self._pass_1_length[idx] == nop_length
                 ):
                     replace_idx = idx
                     break
@@ -657,7 +801,7 @@ class AnalyzingDisassembler(Disassembler):
         # is a copy of _pass_1_insn, but replace the instruction we
         # identified above with a series of 'nop' instructions.
         self._check = list(self._pass_1_insn)
-        nop_count = int(self._pass_1_length[replace_idx] / self._pass_1_length[nop_idx])
+        nop_count = int(self._pass_1_length[replace_idx] / nop_length)
         nop_insn = self._pass_1_insn[nop_idx]
         nops = [nop_insn] * nop_count
         self._check[replace_idx : (replace_idx + 1)] = nops
@@ -696,7 +840,6 @@ class InvalidDisassembleInfo(gdb.disassembler.DisassembleInfo):
 
     @property
     def address(self):
-        global current_pc
         return current_pc
 
     @property
